package com.sunyard.chsm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sunyard.chsm.dto.KeyInfoDTO;
import com.sunyard.chsm.enums.KeyCategory;
import com.sunyard.chsm.enums.KeyStatus;
import com.sunyard.chsm.enums.KeyUsage;
import com.sunyard.chsm.mapper.KeyInfoMapper;
import com.sunyard.chsm.mapper.KeyTemplateMapper;
import com.sunyard.chsm.mapper.SpKeyRecordMapper;
import com.sunyard.chsm.model.entity.KeyInfo;
import com.sunyard.chsm.model.entity.KeyRecord;
import com.sunyard.chsm.model.entity.KeyTemplate;
import com.sunyard.chsm.sdf.SdfApiService;
import com.sunyard.chsm.sdf.model.EccKey;
import com.sunyard.chsm.service.KeyInfoService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author liulu
 * @since 2024/10/23
 */
@Slf4j
@Service
public class KeyInfoServiceImpl implements KeyInfoService {

    @Resource
    private KeyInfoMapper keyInfoMapper;
    @Resource
    private SpKeyRecordMapper spKeyRecordMapper;
    @Resource
    private KeyTemplateMapper keyTemplateMapper;
    @Resource
    private SdfApiService sdfApiService;


    @Override
    public Page<KeyInfoDTO.View> selectPageList(KeyInfoDTO.Query query) {

        IPage<KeyInfo> page = keyInfoMapper.selectPage(
                new Page<>(query.getPageNumber(), query.getPageSize()),
                new LambdaQueryWrapper<KeyInfo>()
                        .eq(StringUtils.hasText(query.getKeyType()), KeyInfo::getKeyType, query.getKeyType())
                        .eq(StringUtils.hasText(query.getStatus()), KeyInfo::getStatus, query.getStatus())
        );
        List<KeyInfo> records = page.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
        }
        List<KeyInfoDTO.View> viewList = records.stream()
                .map(it -> {
                    KeyInfoDTO.View view = new KeyInfoDTO.View();
                    BeanUtils.copyProperties(it, view);
                    Optional.ofNullable(KeyCategory.of(it.getKeyType()))
                            .map(KeyCategory::getDesc)
                            .ifPresent(view::setKeyTypeText);
                    Map<String, String> usageMap = KeyUsage.getUsage(it.getKeyUsage())
                            .stream()
                            .collect(Collectors.toMap(KeyUsage::getCode, KeyUsage::getDesc));
                    view.setKeyUsages(new ArrayList<>(usageMap.keySet()));
                    view.setKeyUsageText(String.join(",", usageMap.values()));
                    Optional.ofNullable(KeyStatus.of(it.getStatus()))
                            .map(KeyStatus::getDesc)
                            .ifPresent(view::setStatusText);

                    return view;
                })
                .collect(Collectors.toList());

        return new Page<KeyInfoDTO.View>(page.getCurrent(), page.getSize(), page.getTotal()).setRecords(viewList);
    }

    @Transactional
    @Override
    public Long save(KeyInfoDTO.Save save) {

        KeyTemplate keyTemplate = keyTemplateMapper.selectOne(
                new LambdaQueryWrapper<KeyTemplate>()
                        .eq(KeyTemplate::getCode, save.getKeyTemplateCode())
        );
        Assert.notNull(keyTemplate, "密钥模版不存在");

        LocalDateTime now = LocalDateTime.now();

        for (int i = 0; i < save.getGenNumber(); i++) {

            // 密钥信息
            KeyInfo info = new KeyInfo();
            info.setId(IdWorker.getId());
            info.setApplicationId(save.getApplicationId());
            info.setKeyTemplateId(keyTemplate.getId());
            info.setCode(String.valueOf(info.getId()));
            info.setKeyType(keyTemplate.getKeyType());
            info.setKeyAlg(keyTemplate.getKeyAlg());
            info.setKeyLength(keyTemplate.getKeyLength());
            info.setKeyUsage(keyTemplate.getKeyUsage());
            info.setCheckAlg(keyTemplate.getCheckAlg());
            info.setCreateTime(LocalDateTime.now());

            info.setStatus(KeyStatus.ENABLED.getCode());

            ChronoUnit startUnit = ChronoUnit.valueOf(keyTemplate.getStartAfterCreateUnit().toUpperCase());
            info.setEffectiveTime(now.plusSeconds(startUnit.getDuration().getSeconds() * keyTemplate.getStartAfterCreateTime()));

            ChronoUnit validUnit = ChronoUnit.valueOf(keyTemplate.getValidUnit().toUpperCase());
            info.setExpiredTime(info.getEffectiveTime().plusSeconds(validUnit.getDuration().getSeconds() * keyTemplate.getValidTime()));

            // 具体使用的密钥值
            KeyRecord record = new KeyRecord();
            record.setId(IdWorker.getId());
            record.setKeyId(info.getId());
            record.setKeyIndex(String.valueOf(record.getId()));
            record.setCheckAlg(keyTemplate.getCheckAlg());
            record.setEffectiveTime(info.getEffectiveTime());
            record.setExpiredTime(info.getExpiredTime());
            record.setCreateTime(LocalDateTime.now());

            if (KeyCategory.SYM_KEY.getCode().equals(keyTemplate.getKeyType())) {
                byte[] symKey = sdfApiService.generateRandom(16);
                record.setKeyData(Hex.encodeHexString(symKey));
                String checkHash = Hex.encodeHexString(sdfApiService.hash(symKey));
                record.setCheckValue(checkHash);
                info.setCheckValue(checkHash);
            } else {
                EccKey eccKey = sdfApiService.genKeyPairEcc();
                byte[] d = eccKey.getPriKey().getD();
                record.setKeyData(Hex.encodeHexString(d));
                String checkHash = Hex.encodeHexString(sdfApiService.hash(d));
                record.setCheckValue(checkHash);
                info.setCheckValue(checkHash);

                byte[] pubKeyBytes = eccKey.getPubKey().getPubKeyBytes();
                record.setPubKey(Hex.encodeHexString(pubKeyBytes));
            }
            keyInfoMapper.insert(info);
            spKeyRecordMapper.insert(record);
        }

        return 0L;
    }
}
